/*
 *  Copyright (C) 2011  Alexandre Courbot
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __GUI_TJLINEEDIT_H
#define __GUI_TJLINEEDIT_H

#include <QLineEdit>

class QToolButton;

/**
 * An improved LineEdit with a clear button.
 * 
 * Largely inspired by Girish Ramakrishnan's example.
 * See http://labs.qt.nokia.com/2007/06/06/lineedit-with-a-clear-button/ for
 * the original.
 */
class TJLineEdit : public QLineEdit
{
	Q_OBJECT
private:
	QToolButton *_resetButton;

protected:
	void resizeEvent(QResizeEvent *event);

public:
	TJLineEdit(QWidget *parent = 0);

private slots:
	void updateResetButton(const QString &text);

};

#endif
