// Autogenerated file, do not edit.
DICTIONARY_DESCRIPTION("busy_people", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Japanese For Busy People\" vols I-III, published by the AJLT. The codes are the volume.chapter."));
DICTIONARY_DESCRIPTION("crowley", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"The Kanji Way to Japanese Language Power\" by Dale Crowley."));
DICTIONARY_DESCRIPTION("gakken", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"A New Dictionary of Kanji Usage\" (Gakken)"));
DICTIONARY_DESCRIPTION("halpern_kkd", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Kodansha Kanji Dictionary\", (2nd Ed. of the NJECD) edited by Jack Halpern."));
DICTIONARY_DESCRIPTION("halpern_kkld", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Kanji Learners Dictionary\" (Kodansha) edited by Jack Halpern."));
DICTIONARY_DESCRIPTION("halpern_kkld_2ed", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Kanji Learners Dictionary\" (Kodansha), 2nd edition (2013) edited by Jack Halpern."));
DICTIONARY_DESCRIPTION("halpern_njecd", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"New Japanese-English Character Dictionary\", edited by Jack Halpern."));
DICTIONARY_DESCRIPTION("heisig", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Remembering The Kanji\" by James Heisig."));
DICTIONARY_DESCRIPTION("heisig6", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Remembering The Kanji, Sixth Ed.\" by James Heisig."));
DICTIONARY_DESCRIPTION("henshall", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"A Guide To Remembering Japanese Characters\" by Kenneth G. Henshall."));
DICTIONARY_DESCRIPTION("henshall3", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"A Guide To Reading and Writing Japanese\" 3rd edition, edited by Henshall, Seeley and De Groot."));
DICTIONARY_DESCRIPTION("jf_cards", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "Japanese Kanji Flashcards, by Max Hodges and Tomoko Okazaki. (Series 1)"));
DICTIONARY_DESCRIPTION("kanji_in_context", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Kanji in Context\" by Nishiguchi and Kono."));
DICTIONARY_DESCRIPTION("kodansha_compact", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "the \"Kodansha Compact Kanji Guide\"."));
DICTIONARY_DESCRIPTION("maniette", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "codes from Yves Maniette's \"Les Kanjis dans la tete\" French adaptation of Heisig."));
DICTIONARY_DESCRIPTION("moro", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Daikanwajiten\" compiled by Morohashi. For some kanji two additional attributes are used: m_vol: the volume of the dictionary in which the kanji is found, and m_page: the page number in the volume."));
DICTIONARY_DESCRIPTION("nelson_c", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Modern Reader's Japanese-English Character Dictionary\", edited by Andrew Nelson (now published as the \"Classic\" Nelson)."));
DICTIONARY_DESCRIPTION("nelson_n", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"The New Nelson Japanese-English Character Dictionary\", edited by John Haig."));
DICTIONARY_DESCRIPTION("oneill_kk", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Essential Kanji\" by P.G. O'Neill."));
DICTIONARY_DESCRIPTION("oneill_names", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Japanese Names\", by P.G. O'Neill."));
DICTIONARY_DESCRIPTION("sakade", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"A Guide To Reading and Writing Japanese\" edited by Florence Sakade."));
DICTIONARY_DESCRIPTION("sh_kk", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Kanji and Kana\" by Spahn and Hadamitzky."));
DICTIONARY_DESCRIPTION("sh_kk2", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "\"Kanji and Kana\" by Spahn and Hadamitzky (2011 edition)."));
DICTIONARY_DESCRIPTION("tutt_cards", QT_TRANSLATE_NOOP("Kanjidic2EntryFormatter", "Tuttle Kanji Cards, compiled by Alexander Kask."));
