/*
 *  Copyright (C) 2009  Alexandre Courbot
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __GUI_JMDICT_YESNOTRAINER_H
#define __GUI_JMDICT_YESNOTRAINER_H

#include "gui/YesNoTrainer.h"

#include <QCheckBox>

class JMdictYesNoTrainer : public YesNoTrainer
{
	Q_OBJECT
private:
	QCheckBox *_showFuriganas;
	QAction *_showFuriganasAction;
public:
	JMdictYesNoTrainer(QWidget *parent);
	virtual void setTrainingMode(TrainingMode mode);

public slots:
	void onShowFuriganasChecked(bool checked);
};

#endif
