table {
	margin: 0px;
	padding: 0px;
}

tr {
	margin: 0px;
	padding: 0px;
}

td {
	margin: 0px;
	padding: 0px;
}

.subinfo {
	width: 100%;
}

.subinfo .title {
	font-weight: bold;
}

.notes {
	width: 100%;
	padding: 5px;
}
